#ifdef _vertex_

layout(location = 0) in vec4 _attr_position_;
layout(location = 1) in vec3 _attr_normal_;
layout(location = 2) in vec2 _attr_texcoord_;
layout(location = 3) in vec3 _attr_tangent_;
layout(location = 4) in vec3 _attr_bitangent_;

uniform mat4 _proj_view_model_matrix_;
uniform mat4 _model_matrix_;
uniform mat3 _normal_matrix_;

out vec2 zw;
out vec3 normal;
out vec2 texcoord;
out vec3 tangent;
out vec3 bitangent;

void main() {
    normal = normalize(_normal_matrix_ * _attr_normal_);
    tangent = normalize(_normal_matrix_ * _attr_tangent_);
    bitangent = normalize(_normal_matrix_ * _attr_bitangent_);
    texcoord = _attr_texcoord_;

    vec4 mvp_position = _proj_view_model_matrix_ * _attr_position_;
    zw = mvp_position.zw;

    gl_Position = mvp_position;
}

#endif

#ifdef _fragment_

layout(location = 0) out vec4 materialDepthTexture;
layout(location = 1) out vec4 normalTexture;
layout(location = 2) out vec4 colorTexture;

uniform sampler2D _diffuse_texture_;
uniform sampler2D _normal_texture_;
uniform sampler2D _lightmap_texture_;

in vec2 zw;
in vec3 normal;
in vec2 texcoord;
in vec3 tangent;
in vec3 bitangent;

void main() {
    vec3 bump = 2.0 * texture(_normal_texture_, texcoord).rgb - vec3(1.0);

    materialDepthTexture = vec4(0.0, 0.0, 0.0, zw.x / zw.y);
    normalTexture = vec4(normalize(bump.x * tangent + bump.y * bitangent + bump.z * normal), 0.0);
    colorTexture = pow(texture(_diffuse_texture_, texcoord), vec4(2.2));
}

#endif
