#ifdef _vertex_

layout(location = 0) in vec4 _attr_position_;
layout(location = 1) in vec2 _attr_texcoord_;

out vec2 texcoord;

void main() {
    gl_Position = _attr_position_;
    texcoord = _attr_texcoord_;
}

#endif

#ifdef _fragment_

layout(location = 0) out vec4 color;

uniform mat4 _inv_proj_view_matrix_;
uniform vec3 _camera_position_;
uniform vec3 _light_position_;
uniform vec3 _light_attenuation_;

uniform sampler2D _material_depth_texture_;
uniform sampler2D _normal_texture_;
uniform sampler2D _color_texture_;

in vec2 texcoord;

void main() {
    vec4 materialDepth = texture(_material_depth_texture_, texcoord);
    float depth = materialDepth.a;
    vec4 position4 = _inv_proj_view_matrix_ *  vec4(texcoord.x * 2.0 - 1.0, texcoord.y * 2.0 - 1.0, depth, 1.0);
    vec3 position = position4.xyz / position4.w;

    vec4 normal_data = texture(_normal_texture_, texcoord);
    vec3 normal = normal_data.xyz;
    vec3 light_dir = _light_position_ - position;
    float light_dist = length(light_dir);
    float light_att = 1.0 / (_light_attenuation_.x + _light_attenuation_.y * light_dist + _light_attenuation_.z * light_dist * light_dist);
    vec3 view_dir  = normalize(_camera_position_ - position);
    vec3 light_half  = normalize(normalize(light_dir) + view_dir);
    float light_spec = pow(max(0.0, dot(light_half, normal)), 32.0) * light_att * 0.5;
    light_spec = min(1.0, light_spec);
    float light_shading = max(dot(normal, normalize(light_dir)), 0.05) * light_att;

    color = light_shading * texture(_color_texture_, texcoord) + light_spec;

    color = pow(color, vec4(1.0 / 2.2));
}

#endif
